@@ -21,7 +21,7 @@ from oslo_log import log
 LOG = log.getLogger(__name__)
 
 
-__all__ = ['Server', 'httplib', 'subprocess']
+__all__ = ('Server', 'httplib', 'subprocess')
 
 _configured = False
 
@@ -95,7 +95,8 @@ def use_stdlib():
     global httplib, subprocess
 
     import six.moves.http_client as _httplib
-    import subprocess as _subprocess
+    import subprocess as _subprocess  # nosec : This is used in .federation.idp
+    # and .common.openssl. See there.
 
     httplib = _httplib
     subprocess = _subprocess
