@@ -58,7 +58,7 @@ class CredentialDriverV8(object):
     def create_credential(self, credential_id, credential):
         """Creates a new credential.
 
-        :raises: keystone.exception.Conflict
+        :raises keystone.exception.Conflict: If a duplicate credential exists.
 
         """
         raise exception.NotImplemented()  # pragma: no cover
@@ -77,10 +77,11 @@ class CredentialDriverV8(object):
         raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
-    def list_credentials_for_user(self, user_id):
+    def list_credentials_for_user(self, user_id, type=None):
         """List credentials for a user.
 
         :param user_id: ID of a user to filter credentials by.
+        :param type: type of credentials to filter on.
 
         :returns: a list of credential_refs or an empty list.
 
@@ -92,7 +93,8 @@ class CredentialDriverV8(object):
         """Get a credential by ID.
 
         :returns: credential_ref
-        :raises: keystone.exception.CredentialNotFound
+        :raises keystone.exception.CredentialNotFound: If credential doesn't
+            exist.
 
         """
         raise exception.NotImplemented()  # pragma: no cover
@@ -101,8 +103,9 @@ class CredentialDriverV8(object):
     def update_credential(self, credential_id, credential):
         """Updates an existing credential.
 
-        :raises: keystone.exception.CredentialNotFound,
-                 keystone.exception.Conflict
+        :raises keystone.exception.CredentialNotFound: If credential doesn't
+            exist.
+        :raises keystone.exception.Conflict: If a duplicate credential exists.
 
         """
         raise exception.NotImplemented()  # pragma: no cover
@@ -111,7 +114,8 @@ class CredentialDriverV8(object):
     def delete_credential(self, credential_id):
         """Deletes an existing credential.
 
-        :raises: keystone.exception.CredentialNotFound
+        :raises keystone.exception.CredentialNotFound: If credential doesn't
+            exist.
 
         """
         raise exception.NotImplemented()  # pragma: no cover
