@@ -27,6 +27,7 @@ from keystone.common.ldap import core as common_ldap_core
 from keystone.tests import unit
 from keystone.tests.unit import default_fixtures
 from keystone.tests.unit import fakeldap
+from keystone.tests.unit.ksfixtures import database
 
 
 CONF = cfg.CONF
@@ -195,8 +196,8 @@ class DnCompareTest(unit.BaseTestCase):
 
     def test_startswith_unicode(self):
         # dn_startswith accepts unicode.
-        child = u'cn=cn=fäké,ou=OpenStäck'
-        parent = 'ou=OpenStäck'
+        child = u'cn=fäké,ou=OpenStäck'
+        parent = u'ou=OpenStäck'
         self.assertTrue(ks_ldap.dn_startswith(child, parent))
 
 
@@ -207,6 +208,8 @@ class LDAPDeleteTreeTest(unit.TestCase):
 
         ks_ldap.register_handler('fake://',
                                  fakeldap.FakeLdapNoSubtreeDelete)
+        self.useFixture(database.Database(self.sql_driver_version_overrides))
+
         self.load_backends()
         self.load_fixtures(default_fixtures)
 
@@ -226,11 +229,11 @@ class LDAPDeleteTreeTest(unit.TestCase):
         config_files.append(unit.dirs.tests_conf('backend_ldap.conf'))
         return config_files
 
-    def test_deleteTree(self):
+    def test_delete_tree(self):
         """Test manually deleting a tree.
 
         Few LDAP servers support CONTROL_DELETETREE.  This test
-        exercises the alternate code paths in BaseLdap.deleteTree.
+        exercises the alternate code paths in BaseLdap.delete_tree.
 
         """
         conn = self.identity_api.user.get_connection()
@@ -251,7 +254,7 @@ class LDAPDeleteTreeTest(unit.TestCase):
         # cn=base
         # cn=child,cn=base
         # cn=grandchild,cn=child,cn=base
-        # then attempt to deleteTree(cn=base)
+        # then attempt to delete_tree(cn=base)
         base_id = 'base'
         base_dn = create_entry(base_id)
         child_dn = create_entry('child', base_dn)
@@ -273,8 +276,8 @@ class LDAPDeleteTreeTest(unit.TestCase):
         self.assertRaises(ldap.NOT_ALLOWED_ON_NONLEAF,
                           conn.delete_s, child_dn)
 
-        # call our deleteTree implementation
-        self.identity_api.user.deleteTree(base_id)
+        # call our delete_tree implementation
+        self.identity_api.user.delete_tree(base_id)
         self.assertRaises(ldap.NO_SUCH_OBJECT,
                           conn.search_s, base_dn, ldap.SCOPE_BASE)
         self.assertRaises(ldap.NO_SUCH_OBJECT,
@@ -283,6 +286,24 @@ class LDAPDeleteTreeTest(unit.TestCase):
                           conn.search_s, grandchild_dn, ldap.SCOPE_BASE)
 
 
+class MultiURLTests(unit.TestCase):
+    """Tests for setting multiple LDAP URLs."""
+
+    def test_multiple_urls_with_comma_no_conn_pool(self):
+        urls = 'ldap://localhost,ldap://backup.localhost'
+        self.config_fixture.config(group='ldap', url=urls, use_pool=False)
+        base_ldap = ks_ldap.BaseLdap(CONF)
+        ldap_connection = base_ldap.get_connection()
+        self.assertEqual(urls, ldap_connection.conn.conn._uri)
+
+    def test_multiple_urls_with_comma_with_conn_pool(self):
+        urls = 'ldap://localhost,ldap://backup.localhost'
+        self.config_fixture.config(group='ldap', url=urls, use_pool=True)
+        base_ldap = ks_ldap.BaseLdap(CONF)
+        ldap_connection = base_ldap.get_connection()
+        self.assertEqual(urls, ldap_connection.conn.conn_pool.uri)
+
+
 class SslTlsTest(unit.TestCase):
     """Tests for the SSL/TLS functionality in keystone.common.ldap.core."""
 
@@ -359,6 +380,7 @@ class LDAPPagedResultsTest(unit.TestCase):
 
         ks_ldap.register_handler('fake://', fakeldap.FakeLdap)
         self.addCleanup(common_ldap_core._HANDLERS.clear)
+        self.useFixture(database.Database(self.sql_driver_version_overrides))
 
         self.load_backends()
         self.load_fixtures(default_fixtures)
