@@ -86,19 +86,7 @@ class TestAssertingNoneEquality(BaseStyleCheck):
         self.assert_has_errors(code, expected_errors=errors)
 
 
-class TestCheckForDebugLoggingIssues(BaseStyleCheck):
-
-    def get_checker(self):
-        return checks.CheckForLoggingIssues
-
-    def test_for_translations(self):
-        fixture = self.code_ex.assert_no_translations_for_debug_logging
-        code = fixture['code']
-        errors = fixture['expected_errors']
-        self.assert_has_errors(code, expected_errors=errors)
-
-
-class TestCheckForNonDebugLoggingIssues(BaseStyleCheck):
+class BaseLoggingCheck(BaseStyleCheck):
 
     def get_checker(self):
         return checks.CheckForLoggingIssues
@@ -106,21 +94,43 @@ class TestCheckForNonDebugLoggingIssues(BaseStyleCheck):
     def get_fixture(self):
         return hacking_fixtures.HackingLogging()
 
+    def assert_has_errors(self, code, expected_errors=None):
+
+        # pull out the parts of the error that we'll match against
+        actual_errors = (e[:3] for e in self.run_check(code))
+        # adjust line numbers to make the fixture data more readable.
+        import_lines = len(self.code_ex.shared_imports.split('\n')) - 1
+        actual_errors = [(e[0] - import_lines, e[1], e[2])
+                         for e in actual_errors]
+        self.assertEqual(expected_errors or [], actual_errors)
+
+
+class TestCheckForDebugLoggingIssues(BaseLoggingCheck):
+
+    def test_for_translations(self):
+        fixture = self.code_ex.assert_no_translations_for_debug_logging
+        code = self.code_ex.shared_imports + fixture['code']
+        errors = fixture['expected_errors']
+        self.assert_has_errors(code, expected_errors=errors)
+
+
+class TestLoggingWithWarn(BaseLoggingCheck):
+
+    def test(self):
+        data = self.code_ex.assert_not_using_deprecated_warn
+        code = self.code_ex.shared_imports + data['code']
+        errors = data['expected_errors']
+        self.assert_has_errors(code, expected_errors=errors)
+
+
+class TestCheckForNonDebugLoggingIssues(BaseLoggingCheck):
+
     def test_for_translations(self):
         for example in self.code_ex.examples:
             code = self.code_ex.shared_imports + example['code']
             errors = example['expected_errors']
             self.assert_has_errors(code, expected_errors=errors)
 
-    def assert_has_errors(self, code, expected_errors=None):
-        # pull out the parts of the error that we'll match against
-        actual_errors = (e[:3] for e in self.run_check(code))
-        # adjust line numbers to make the fixure data more readable.
-        import_lines = len(self.code_ex.shared_imports.split('\n')) - 1
-        actual_errors = [(e[0] - import_lines, e[1], e[2])
-                         for e in actual_errors]
-        self.assertEqual(expected_errors or [], actual_errors)
-
 
 class TestDictConstructorWithSequenceCopy(BaseStyleCheck):
 
